local slowThrottle = 0.50 -- update speed when not over a unit frame
local midThrottle = 0.10 -- update speed while over a unit that is far away
local fastThrottle = 0.01 -- update speed while unit is relatively near

local pointer = CreateFrame("Frame",nil,UIParent)
pointer:Hide()
pointer.timer = 0
pointer:SetFrameStrata("FULLSCREEN")

pointer.arrow = CreateFrame("Frame",nil,pointer)
pointer.arrow.texture = pointer.arrow:CreateTexture(nil,"OVERLAY")
pointer.arrow.texture:SetAllPoints(true)
pointer.arrow.texture:SetTexture("Interface\\MiniMap\\MiniMap-QuestArrow")
pointer.arrow:SetSize(32,32)

pointer.yards = pointer.arrow:CreateFontString(nil,"OVERLAY","GameFontHighlightSmall")
pointer.yards:SetPoint("RIGHT",pointer.arrow,"LEFT")

local in_bg

local sqrt2 = sqrt(2) -- stuff to help texcoord mumbo jumbo later
local rads45 = 0.25*PI
local rads135 = 0.75*PI
local rads225 = 1.25*PI
local cos, sin = math.cos, math.sin
local function corner(r) return 0.5+cos(r)/sqrt2, 0.5+sin(r)/sqrt2 end

-- self here is pointer.arrow; update whether to show arrow and direction it points
local function UpdateArrow(self)

	-- see if there's a unit frame under the mouse
	local focus = GetMouseFocus()
	local unit = focus and not focus:IsForbidden() and focus:GetName() and focus:GetName():match("CompactRaidFrame") and focus:GetAttribute("unit")
	if not unit then
		self:Hide()
		pointer.timer = slowThrottle
		return
	end

	local playerX, playerY = UnitPosition("player")
	local unitX, unitY = UnitPosition(unit)

	-- UnitInRange() is what dims default raid frames; it doesn't precisely align with UnitDistanceSquared()
	-- so using UnitInRange() for the more-important check if in spell casting range
	local inRange = UnitInRange(unit)

	-- hide arrow if unit does not have coordinates (not in group, pet, etc), or
	-- player and unit are same unit, or unit is near (and ShowInRange not set in options)
	if not playerX or not unitX or UnitIsUnit("player",unit) or inRange or not in_bg then
		self:Hide()
		pointer.timer = midThrottle
		return
	end

	local distance = UnitDistanceSquared(unit)

	pointer.timer = fastThrottle -- we're going to show an arrow, speed up updates

	local arrow = self.texture

	if distance<22500 then -- less than 150 yards, oor but near: yellow (default color)
		arrow:SetVertexColor(1,1,1)
	else
		arrow:SetVertexColor(1,0,0)
	end

	-- calculate angle between player facing and mouseover map position (trig courtesy of Javonthalas-Hyjal)
	local angle = atan2(unitY-playerY,unitX-playerX)*PI/180 - GetPlayerFacing()

	-- rotate arrow to the calculated angle; from http://wowpedia.org/SetTexCoord_Transformations
	local ULx,ULy = corner(angle+rads225)
	local LLx,LLy = corner(angle+rads135)
	local URx,URy = corner(angle-rads45)
	local LRx,LRy = corner(angle+rads45)
	arrow:SetTexCoord(ULx,ULy,LLx,LLy,URx,URy,LRx,LRy)

	-- position arrow beside cursor
	local cursorX, cursorY = GetCursorPosition()
	local scale = UIParent:GetEffectiveScale()
	self:SetPoint("CENTER",focus,"CENTER")

	pointer.yards:SetText(format("%.0f",sqrt(distance)))

	self:Show()
end

pointer:SetScript("OnUpdate",function(self,elapsed)
	self.timer = self.timer - elapsed
	if self.timer < 0 then
		UpdateArrow(self.arrow)
	end
end)

WorldFrame:HookScript("OnEnter",function() pointer:Hide() end)
WorldFrame:HookScript("OnLeave",function() if IsInGroup() then pointer.timer=0 pointer:Show() end end)

pointer:SetScript("OnEvent",function(self,event,...)
	if event=="PLAYER_ENTERING_WORLD" then
		local _,InstanceType = IsInInstance()
		in_bg = InstanceType=="pvp"
	end
end)
pointer:RegisterEvent("PLAYER_ENTERING_WORLD")